/*
 * Copyright (c) 2007 Tomi Aarnio
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

//----------------------------------------------------------------------
// Retrieve the M3G property values
//----------------------------------------------------------------------

package com.graphicsformasses.ch13;
import com.graphicsformasses.framework.DemoTask;

import java.util.*;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.m3g.Graphics3D;

public class GetProperties extends DemoTask
{
    private boolean redraw;
    private int scrollLines;
    private int scrollTime;
    
    private Hashtable properties = Graphics3D.getProperties();
    private Font font;
    
    protected String getName() {
        return "getProperties";
    }

    protected void start() {
        font = Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_PLAIN, Font.SIZE_SMALL);
        redraw = true;
    }
    
    protected boolean run(int time, int keyStates, int keysChanged, Graphics g) {

        boolean scrollUp   = (keyStates & GameCanvas.UP_PRESSED) != 0;
        boolean scrollDown = (keyStates & GameCanvas.DOWN_PRESSED) != 0;
        
        if (time - scrollTime > 100) {
            if (scrollUp) {
                if (scrollLines > 0) {
                    --scrollLines;
                    redraw = true;
                }
            }
            if (scrollDown) {
                if (scrollLines < properties.size() - 1) {
                    ++scrollLines;
                    redraw = true;
                }
            }
            if (scrollUp || scrollDown) {
                scrollTime = time;
            }
        }
        if (!(scrollUp || scrollDown)) {
            scrollTime = 0;
        }
        
        
        if (redraw) {
            g.setColor(0x00000000);
            g.fillRect(g.getClipX(), g.getClipY(), g.getClipWidth(), g.getClipHeight());
            g.setFont(font);
            g.setColor(0xFF44FF55);
            
            int h = g.getFont().getHeight();
            int y = 2 - scrollLines * h;
            
            Enumeration e = properties.keys();
            while (e.hasMoreElements()) {
                String key = (String) e.nextElement();
                g.drawString(key + ": " + properties.get(key).toString(), 2, y, Graphics.TOP|Graphics.LEFT);
                y += h;
            }
            redraw = false;
        }
        return true;
    }
}
