/*
 * Copyright (c) 2007 Tomi Aarnio
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

//----------------------------------------------------------------------
// Draw in immediate mode
//----------------------------------------------------------------------

package com.graphicsformasses.ch13;
import com.graphicsformasses.framework.DemoTask;

import java.util.*;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.m3g.*;

public class MaxViewport extends DemoTask
{
    private Graphics3D g3d = Graphics3D.getInstance();
    private Hashtable properties = Graphics3D.getProperties();
    private World myWorld;
    private boolean fsaa;
    
    private int maxViewport, ulx, uly;
    private float zoom;
    
    protected String getName() {
        return "Max viewport";
    }

    protected void start() {
        maxViewport = ((Integer)properties.get("maxViewportDimension")).intValue();
        ulx = uly = 0;
        zoom = 1.0f;
        fsaa = true;
        try {
            myWorld = (World)Loader.load("/pond.m3g")[0];
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
    }

    protected boolean run(int time, int keyStates, int keysChanged, Graphics g) {

        int keysPressed = keyStates; // & keysChanged;
        
        // Move the viewport top-left corner according to arrow key presses.
        //
        boolean up = (keyStates & GameCanvas.UP_PRESSED) != 0;
        boolean down = (keyStates & GameCanvas.DOWN_PRESSED) != 0;
        boolean left = (keyStates & GameCanvas.LEFT_PRESSED) != 0;
        boolean right = (keyStates & GameCanvas.RIGHT_PRESSED) != 0;
        boolean zoomin = (keysPressed & GameCanvas.GAME_A_PRESSED) != 0;
        boolean zoomout = (keysPressed & GameCanvas.GAME_B_PRESSED) != 0;
        if (up) uly += 10;
        if (down) uly -= 10;
        if (left) ulx += 10;
        if (right) ulx -= 10;
        if (zoomin && (zoom < 100.f)) zoom *= 1.03f;
        if (zoomout && (zoom > 0.01f)) zoom /= 1.03f;
        
        // Toggle anti-aliasing by pressing the 'fire' key
        //
        boolean fire = (keyStates & GameCanvas.FIRE_PRESSED) != 0;
        if (fire) fsaa = !fsaa;
        int flags = Graphics3D.OVERWRITE;
        if (fsaa) flags |= Graphics3D.ANTIALIAS;

        // Clear the screen, set up the viewport, and render
        //
        g3d.bindTarget(g, true, flags);
        g3d.clear(null);
        
        int viewportWidth = (int)(g3d.getViewportWidth() * zoom);
        int viewportHeight = (int)(g3d.getViewportHeight() * zoom);
        viewportWidth = Math.max(Math.min(maxViewport, viewportWidth), 1);
        viewportHeight = Math.max(Math.min(maxViewport, viewportHeight), 1);
        int topLeftX = -(viewportWidth - g3d.getViewportWidth())/2;
        int topLeftY = -(viewportHeight - g3d.getViewportHeight())/2;
        topLeftX += ulx;
        topLeftY += uly;

        g3d.setViewport(topLeftX, topLeftY, viewportWidth, viewportHeight);
        myWorld.animate(time);
        g3d.render(myWorld);
        g3d.releaseTarget();
        
        return true;
    }
}
