/*
 * Copyright (c) 2007 Kimmo Roimela
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

//----------------------------------------------------------------------
// Colored triangles in immediate mode -- example from Section 14.1.4
//----------------------------------------------------------------------

package com.graphicsformasses.ch14;
import com.graphicsformasses.framework.DemoTask;

import javax.microedition.m3g.*;
import javax.microedition.lcdui.Graphics;

public class ColoredTriangles extends DemoTask
{
    static final byte positions[] = { 0,100,0, 100,0,0, 0,-100,0, -100,0,0,
                                      0,50,0, 45,20,0, -45,20,0 };
    static final byte colors[] = { 0,0,(byte)255,(byte)255, 0,(byte)255,0,(byte)255, (byte)255,0,0,(byte)255,
                                   (byte)255,(byte)255,(byte)255,(byte)255, (byte)255,0,(byte)255,(byte)255,
                                   (byte)255,(byte)255,0,(byte)255, 0,(byte)255,(byte)255,(byte)255 };

    static final int indices[] = { 0,3,1, 1,3,2, 4,6,5 };
    static final int strips[]  = { 3, 3, 3 };

    static final int altIndices[] = { 0,3,1,2, 4,6,5 };
    static final int altStrips[]  = { 4, 3 };

    protected Graphics3D g3d = Graphics3D.getInstance();
    
    protected VertexBuffer myVertices;
    protected IndexBuffer myTriangles;
    protected Appearance myAppearance;
    protected Transform myModelTransform;

    protected String getName() {
        return "Colored triangles";
    }

    protected void resize(int w, int h) {
        Camera camera = new Camera();
        camera.setPerspective(60.0f, (float)w/(float)h, 10.0f, 500.0f);
        g3d.setCamera(camera, null);
    }
    
    protected void start() {

        // Create the vertex arrays
        
        VertexArray myPositionArray = new VertexArray(7, 3, 1);
        VertexArray myColorArray = new VertexArray(7, 4, 1);
        
        // Set values for 7 vertices starting at vertex 0
        
        myPositionArray.set(0, 7, positions);
        myColorArray.set(0, 7, colors);
        
        // Create the vertex buffer; for the vertex positions,
        // we set the scale to 1.0 and the bias to zero
        
        myVertices = new VertexBuffer();
        myVertices.setPositions(myPositionArray, 1.0f, null);
        myVertices.setColors(myColorArray);
        
        // Create the indices for five triangles as explicit triangle strips
        
        myTriangles = new TriangleStripArray(indices, strips);

        // Use the default shading parameters
        
        myAppearance = new Appearance();
        
        // Set up a modeling transformation
        
        myModelTransform = new Transform();
        myModelTransform.postTranslate(0.0f, 0.0f, -150.0f);
    }

    protected boolean run(int time,
                          int keyStates, int keysChanged,
                          Graphics g) {
        Graphics3D g3d = Graphics3D.getInstance();
        try {
            g3d.bindTarget(g);
            g3d.clear(null);
            g3d.render(myVertices, myTriangles, myAppearance, myModelTransform);
        }
        finally {
            g3d.releaseTarget();
        }
        return false;
    }
}
