/*
 * Copyright (c) 2007 Kimmo Roimela
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

//----------------------------------------------------------------------
// Lights and materials -- example from Section 14.3.2
//----------------------------------------------------------------------

package com.graphicsformasses.ch14;
import com.graphicsformasses.framework.DemoTask;

import javax.microedition.m3g.*;
import javax.microedition.lcdui.Graphics;


public class Lighting extends ModifiedAppearance
{
    protected String getName() {
        return "Light and Material";
    }

    static final int red = 0xFFFF0000;
    static final int white = 0xFFFFFFFF;
    static final int red_transp = 0x00FF0000;
    static final int blueish = 0xFF2066FF;
    static final int black = 0;

    /*
    protected void resize(int w, int h) {

        float cameraX = 0, cameraY = 0, cameraZ = 0;
        float cameraYaw = 0, cameraPitch = 0;
        float cameraFOV = 60.f;
        
        Camera myCamera = new Camera();
        
        Transform cameraTransform = new Transform();
        cameraTransform.postTranslate(cameraX, cameraY, cameraZ);
        cameraTransform.postRotate(cameraYaw, 0, -1.f, 0);
        cameraTransform.postRotate(cameraPitch, 1.f, 0, 0);
        g3d.setCamera(myCamera, cameraTransform);
        
        float width = w;
        float height = h;
        if (height >= width) {
            myCamera.setPerspective(cameraFOV, width/height, 1.0f, 100.0f);
        }
        else {
            myCamera.setPerspective(cameraFOV*height/width, width/height,
                                    1.0f, 100.0f);
        }
    }
    */
        
    protected void start() {
        
        super.start();
            
        // Create a scene global ambient light
        
        Light ambient = new Light();
        ambient.setMode(Light.AMBIENT);
        ambient.setColor(red);
        ambient.setIntensity(0.2f);
        g3d.addLight(ambient, null);
        
        // Create a directional light at the origin, shining in the direction
        // of the negative Z axis
        
        Light dirLight = new Light();
        dirLight.setMode(Light.DIRECTIONAL);
        dirLight.setColor(white);
        dirLight.setIntensity(1.0f);
        dirLight.setAttenuation(1.f, 0.f, 0.f);
        g3d.addLight(dirLight, null);
        
        // Create a spot light close to the origin, aimed diagonally down
        // and to the left -- note that in immediate mode, the transformation
        // in the Light object is ignored, so we pass it as a separate object
        
        Light spotLight = new Light();
        spotLight.setTranslation(120.f, 120.f, 0.f);
        spotLight.setOrientation(60.f, -1.f, 1.f, 0.f);
        spotLight.setMode(Light.SPOT);
        spotLight.setColor(white);
        spotLight.setIntensity(5.0f);
        spotLight.setSpotAngle(40.f);
        spotLight.setSpotExponent(10.f);
        spotLight.setAttenuation(0.f, 0.01f, 0.f);
        Transform t = new Transform();
        spotLight.getCompositeTransform(t);
        g3d.addLight(spotLight, t);
        
        // Create a material to receive the lights
        
        Material material = new Material();
        material.setColor(Material.AMBIENT|Material.DIFFUSE, blueish);
        material.setColor(Material.SPECULAR, red_transp);
        material.setColor(Material.EMISSIVE, black);
        material.setShininess(15.f);
        myAppearance.setMaterial(material);
    }

    protected boolean run(int time,
                          int keyStates, int keysChanged,
                          Graphics g) {
        try {
            g3d.bindTarget(g);
            g3d.clear(null);
            myModelTransform.setIdentity();
            myModelTransform.postTranslate(0.0f, 0.0f, -150.0f);
            myModelTransform.postRotate(0.1f * time, 0, -1.0f, 0);
            g3d.render(myVertices, myTriangles, myAppearance, myModelTransform);
        }
        finally {
            g3d.releaseTarget();
        }
        return true;
    }
}
