/*
 * Copyright (c) 2007 Kimmo Roimela
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

//----------------------------------------------------------------------
// Colored triangles in immediate mode -- example from Section 14.1.4
//----------------------------------------------------------------------

package com.graphicsformasses.ch14;
import com.graphicsformasses.framework.DemoTask;

import javax.microedition.m3g.*;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

public class ModifiedAppearance extends ColoredTriangles
{
    static final byte normals[] = { 0,0,127, 0,0,127, 0,0,127, 0,0,127,
                                    0,0,127, 0,0,127, 0,0,127 };
    
    private Material redPlastic, golden;
    
    private int width, height;
    
    protected String getName() {
        return "Modified Appearance";
    }
    
    protected void start() {
        
        super.start();

        // Add normal vectors and texture coordinates
        
        VertexArray normalArray = new VertexArray(7, 3, 1);
        normalArray.set(0, 7, normals);
        myVertices.setNormals(normalArray);
        myVertices.setTexCoords(0, myVertices.getPositions(null), 0.01f, null);
        
        // PolygonMode
        
        PolygonMode myPolygonMode = new PolygonMode();
        myAppearance.setPolygonMode(myPolygonMode);
        myPolygonMode.setCulling(PolygonMode.CULL_NONE);
        myPolygonMode.setTwoSidedLightingEnable(true);
        myPolygonMode.setPerspectiveCorrectionEnable(true);
        myPolygonMode.setLocalCameraLightingEnable(true);

        // Texture2D

        // Create a repeating texture image to multiply with the incoming color

        Image2D brickImage;
        try {
            brickImage = new Image2D(Image2D.RGB, Image.createImage("/brick.png"));
        }
        catch (Exception e) {
            brickImage = new Image2D(Image2D.RGB, 64, 64);
        }
        
        Texture2D myTexture = new Texture2D(brickImage);
        myTexture.setWrapping(Texture2D.WRAP_REPEAT, Texture2D.WRAP_REPEAT);
        myTexture.setBlending(Texture2D.FUNC_MODULATE);
        myTexture.setFiltering(Texture2D.FILTER_NEAREST,
                               Texture2D.FILTER_NEAREST);
        
        // Set as the first texture to an Appearance object created earlier;
        // the other texture slots are assumed to be null
        
        myAppearance.setTexture(0, myTexture);

        // Fog

        Fog myFog = new Fog();
        myAppearance.setFog(myFog);
        myFog.setMode(Fog.EXPONENTIAL);
        myFog.setDensity(0.001f);
        myFog.setColor(0x6688FF); // pale blue tint

        // CompositingMode
        
        CompositingMode myCompositingMode = new CompositingMode();
        myAppearance.setCompositingMode(myCompositingMode);
        myCompositingMode.setAlphaThreshold(0.5f);
        myCompositingMode.setDepthOffset(-1.0f, -1.0f);
    }
}
