/*
 * Copyright (c) 2007 Kimmo Roimela
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

//----------------------------------------------------------------------
// Scene graph example, Chapter 15
//----------------------------------------------------------------------

package com.graphicsformasses.ch15;
import com.graphicsformasses.framework.DemoTask;
import com.graphicsformasses.tools.ShapeFactory;

import javax.microedition.m3g.*;
import javax.microedition.lcdui.Graphics;

public class Car extends DemoTask
{
    private Graphics3D g3d = Graphics3D.getInstance();
    
    private World myCarScene = new World();
    private Group myCar;
    private Camera camera = new Camera();

    protected String getName() {
        return "Car";
    }
    
    protected void resize(int w, int h) {
        camera.setPerspective(60.0f, (float)w/(float)h, 1.0f, 1000.0f);
    }

    protected void start() {
        
        Mesh myCarBody;
        
        Mesh[] myCarWheel = new Mesh[4];
        float[] wheelX = {-20.f, 20.f, -22.f, 22.f};
        float[] wheelY = {-35.f, -35.f, 35.f, 35.f};
        float[] wheelZ = {-5.f, -5.f, -4.f, -4.f};
            
        Light spotLight = new Light();
        Background myBackground = new Background();

        myCarBody = ShapeFactory.makeCylinder(25.f, 100.f, 50);
        myCarWheel[0] = ShapeFactory.makeCylinder(25.f, 8.f, 50);
        myCarWheel[1] = (Mesh) myCarWheel[0].duplicate();
        myCarWheel[2] = ShapeFactory.makeCylinder(27.f, 10.f, 50);
        myCarWheel[3] = (Mesh) myCarWheel[2].duplicate();
            
        // Some initialization code would go here -- omitted for brevity

        Material wheelMaterial = new Material();
        wheelMaterial.setColor(Material.DIFFUSE|Material.AMBIENT, 0x333333);
        wheelMaterial.setColor(Material.SPECULAR, 0x888888);
        wheelMaterial.setShininess(40.f);
        myCarWheel[0].getAppearance(0).setMaterial(wheelMaterial);
        myCarWheel[2].getAppearance(0).setMaterial(wheelMaterial);

        Material bodyMaterial = new Material();
        bodyMaterial.setColor(Material.DIFFUSE|Material.AMBIENT, 0xEE3322);
        bodyMaterial.setColor(Material.SPECULAR, 0xFFEECC);
        bodyMaterial.setShininess(100.f);
        myCarBody.getAppearance(0).setMaterial(bodyMaterial);

        myBackground.setColor(0x006699FF);
        
        myCarScene.setBackground(myBackground);
        myCar = new Group();
        myCar.addChild(myCarBody);
        myCarBody.setTranslation(0.f, -5.f, 0.f);
        myCarBody.setScale(1.20f, 0.9f, 1.f);
        myCarBody.setOrientation(90.f, 1.f, 0.f, 0.f);
        for (int i = 0; i < 4; ++i) {
            myCar.addChild(myCarWheel[i]);
            myCarWheel[i].setTranslation(wheelX[i], wheelY[i], wheelZ[i]);
            myCarWheel[i].setOrientation(90.f, 0.f, 1.f, 0.f);
        }
        
        myCarScene.addChild(myCar);
        myCar.setScale(0.1f, 0.1f, 0.1f);
        myCar.setOrientation(-90.f, 1.f, 0.f, 0.f);

        myCarScene.addChild(spotLight);
        spotLight.setTranslation(10.f, 20.f, -30.f);
        spotLight.setOrientation(40.f, -1.f, 1.f, 0.f);
        
        myCarScene.addChild(camera);
        camera.setTranslation(0.f, 3.f, 20.f);
        camera.setOrientation(-10.f, 1.f, 0.f, 0.f);
        myCarScene.setActiveCamera(camera);
    }
    
    protected boolean run(int time, int keys, int keysDelta, Graphics g) {
        Graphics3D g3d = Graphics3D.getInstance();

        myCar.postRotate(3.f, 0.f, 0.f, 1.f);
        
        g3d.bindTarget(g);
        g3d.render(myCarScene);
        g3d.releaseTarget();

        return true;
    }
}
