/*
 * Copyright (c) 2007 Kimmo Roimela
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

//----------------------------------------------------------------------
// Infinitely streaming animation via a KeyframeSequence buffer
//----------------------------------------------------------------------

package com.graphicsformasses.ch16;

import com.graphicsformasses.framework.*;
import com.graphicsformasses.tools.KeyframeStream;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.m3g.*;

import java.util.Random;


public class Streaming extends DemoTask
{
    Graphics3D g3d = Graphics3D.getInstance();
    
    AnimationController animCtrl;
    AnimationTrack animTrk;
    KeyframeSequence kfBuffer;
    KeyframeStream kfStream;

    Sprite3D blob;

    Transform t = new Transform();
        

    public class MotionStream extends KeyframeStream 
    {
        private Random rnd;
        private int streamTime;
        
        public MotionStream(AnimationController ctrl, KeyframeSequence seq) {
            super(ctrl, seq);
            rnd = new Random();
        }

        protected int getNextKeyframe(float[] kf) {
            streamTime += (rnd.nextInt() >> 23) + 1024;  // 768..1280 ms
            kf[0] = (rnd.nextInt() >> 24) / 128.f;
            kf[1] = (rnd.nextInt() >> 24) / 128.f;
            kf[2] = 0.0f;
            return streamTime;
        }
    }
    
    protected String getName() {
        return "Streaming animation";
    }

    protected void start() {
        
        kfBuffer = new KeyframeSequence(6, 3, KeyframeSequence.SPLINE);
        animCtrl = new AnimationController();
        kfStream = new MotionStream(animCtrl, kfBuffer);
        
        animTrk = new AnimationTrack(kfBuffer, AnimationTrack.TRANSLATION);
        animTrk.setController(animCtrl);
        
        Image blobImage;
        try {
            blobImage = Image.createImage("/blob.png");
        }
        catch (Exception e) {
            blobImage = Image.createImage(16, 16);
        }
        
        blob = new Sprite3D(false, new Image2D(Image2D.RGB, blobImage), new Appearance());
        blob.getAppearance().setCompositingMode(new CompositingMode());
        blob.getAppearance().getCompositingMode().setBlending(CompositingMode.ALPHA_ADD);
        blob.addAnimationTrack(animTrk);

        g3d.setCamera(new Camera(), null);
    }

    protected boolean run(int time, int keyStates, int keysChanged, Graphics g) {

        float[] xyz = new float[3];
        
        try {
            g3d.bindTarget(g, false, 0);        // no depth test
            g3d.clear(null);
            kfStream.update(time);
            for (int d = -1000; d <= 0; d += 50) {
                blob.setAlphaFactor((1000 + d) / 1000.f);
                blob.animate(time + d);
                blob.getCompositeTransform(t);
                g3d.render(blob, t);
            }
        }
        finally {
            g3d.releaseTarget();
        }
    
        return true;
    }
}
