/*
 * Copyright (c) 2007 Kimmo Roimela
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.graphicsformasses.framework;

import javax.microedition.lcdui.Display;
import javax.microedition.midlet.*;

/**
 * Base class for MIDlets implementing a collection of demos.
 */
public class DemoMIDlet extends MIDlet
{
    private DemoCanvas canvas;

    /**
     * Returns a new DemoMIDlet instance.
     *
     * @param demopackage  Java package name containing the demos
     * @param demoNames    array of demo class names to display
     *
     * @return new DemoMIDlet instance
     */
    public DemoMIDlet(String demoPackage, String[] demoNames) {
        canvas = new DemoCanvas(this, demoPackage, demoNames);
        Display.getDisplay(this).setCurrent(canvas);
    }

    protected void startApp() {
        canvas.start();
    }
    
    protected void pauseApp() {
        canvas.stop();
        notifyPaused();
    }

    protected void destroyApp(boolean force) {
        canvas.stop();
        canvas = null;
        notifyDestroyed();
    }
}
