/*
 * Copyright (c) 2007 Kimmo Roimela
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.graphicsformasses.framework;

import java.util.Vector;
import javax.microedition.lcdui.Graphics;
import javax.microedition.m3g.Graphics3D;

/**
 * Abstract base class for a single demo.
 */
public abstract class DemoTask
{
    /**
     * Called to retrieve a short, descriptive name for the demo.
     */
    protected abstract String getName();

    /**
     * Called prior to executing the demo and each time the screen
     * size is changed.
     *
     * @param w  new screen width
     * @param h  new screen height
     */
    protected void resize(int w, int h) {
    }

    /**
     * Called prior to executing the demo.
     */
    protected void start() {
    }

    /**
     * Called to execute the demo for one cycle.
     *
     * @param time         current time in milliseconds since start of demo
     * @param keyStates    current key states
     * @param keysChanged  key states changed since last invokation
     * @param g            the Graphics object to draw into
     *
     * @return <code>true</code> to continute running,
     * <code>false</code> to end the demo
     */
    protected abstract boolean run(int time, int keyStates, int keysChanged, Graphics g);

    /**
     * Called when the demo is stopped.
     */
    protected void stop(){
    }
}
